@@ -1,4 +1,5 @@
 import asyncio
+import json
 from functools import wraps
 
 from .flow import CloudFlow
@@ -14,8 +15,11 @@ def asyncify(f):
 
 @asyncify
 async def deploy(args):
-    await CloudFlow(
-        path=args.path, name=args.name, workspace_id=args.workspace
+    return await CloudFlow(
+        path=args.path,
+        name=args.name,
+        workspace_id=args.workspace,
+        env_file=args.env_file,
     ).__aenter__()
 
 
@@ -23,17 +27,22 @@ async def deploy(args):
 async def status(args):
     from rich import box
     from rich.console import Console
+    from rich.json import JSON
     from rich.syntax import Syntax
     from rich.table import Table
 
+    from .helper import CustomHighlighter
+
     _t = Table('Attribute', 'Value', show_header=False, box=box.ROUNDED, highlight=True)
 
-    console = Console()
+    console = Console(highlighter=CustomHighlighter())
     with console.status(f'[bold]Fetching status of {args.flow}...'):
         _result = await CloudFlow(flow_id=args.flow).status
         for k, v in _result.items():
-            if k == 'yaml':
+            if k == 'yaml' and v is not None:
                 v = Syntax(v, 'yaml', theme='monokai', line_numbers=True, code_width=40)
+            elif k == 'envs' and v:
+                v = JSON(json.dumps(v))
             else:
                 v = str(v)
             _t.add_row(k, v)
@@ -42,13 +51,25 @@ async def status(args):
 
 @asyncify
 async def list(args):
+    from datetime import datetime, timezone
+
     from rich import box
     from rich.console import Console
     from rich.table import Table
 
-    _t = Table('ID', 'Status', 'Gateway', box=box.ROUNDED, highlight=True)
+    from .helper import CustomHighlighter
 
-    console = Console()
+    def cleanup(dt) -> str:
+        try:
+            return datetime.strptime(dt, '%Y-%m-%dT%H:%M:%S.%f%z').strftime(
+                '%d-%b-%Y %H:%M'
+            )
+        except:
+            return dt
+
+    _t = Table('ID', 'Status', 'Gateway', 'Created', box=box.ROUNDED, highlight=True)
+
+    console = Console(highlighter=CustomHighlighter())
     _status = args.status
     with console.status(
         f'[bold]Listing flows with status [green]{_status}[/green] ...'
@@ -58,19 +79,25 @@ async def list(args):
         _result = await CloudFlow().list_all(status=args.status)
         if _result:
             for k in _result:
-                _t.add_row(k['id'].split('-')[-1], k['status'], k['gateway'])
+                _t.add_row(
+                    k['id'].split('-')[-1],
+                    k['status'],
+                    k['gateway'],
+                    cleanup(k['ctime']),
+                )
             console.print(_t)
 
 
 @asyncify
 async def remove(args):
+    from rich import print
+
     await CloudFlow(flow_id=args.flow).__aexit__()
+    print(f'Successfully removed Flow [green]{args.flow}[/green]')
 
 
 @asyncify
 async def logs(args):
-    from rich.console import Console
-
     from .flow import pb_task, pbar
 
     with pbar:
@@ -106,7 +133,7 @@ def new(args):
     from rich import print
 
     print(
-        f'[green]New project create under {args.path}[/green]. Try [b]jc deploy {args.path}[/b].'
+        f'[green]New project created under {args.path}[/green]. Try [b]jc deploy {args.path}[/b].'
     )
 
 
