@@ -1,9 +1,9 @@
 import asyncio
 import json
+import logging
 import os
 from contextlib import suppress
 from dataclasses import dataclass
-from enum import Enum
 from http import HTTPStatus
 from pathlib import Path
 from typing import Dict, List, Optional
@@ -11,11 +11,8 @@ from typing import Dict, List, Optional
 import aiohttp
 from rich import print
 
-from .helper import get_logger, get_or_reuse_loop, get_pbar, zipdir
-
-WOLF_API = 'https://api.wolf.jina.ai/dev/flows'
-LOGSTREAM_API = 'wss://logs.wolf.jina.ai/dev'
-ARTIFACT_API = 'https://apihubble.jina.ai/v2/rpc/artifact.upload'
+from .constants import ARTIFACT_API, LOGSTREAM_API, WOLF_API, Status
+from .helper import get_logger, get_or_reuse_loop, get_pbar, normalized, zipdir
 
 logger = get_logger()
 
@@ -24,34 +21,12 @@ pbar, pb_task = get_pbar(
 )  # progress bar for deployment
 
 
-class Status(str, Enum):
-    SUBMITTED = 'SUBMITTED'
-    NORMALIZING = 'NORMALIZING'
-    NORMALIZED = 'NORMALIZED'
-    STARTING = 'STARTING'
-    FAILED = 'FAILED'
-    ALIVE = 'ALIVE'
-    UPDATING = 'UPDATING'
-    DELETING = 'DELETING'
-    DELETED = 'DELETED'
-
-    @property
-    def streamable(self) -> bool:
-        return self in (Status.ALIVE, Status.UPDATING, Status.DELETING)
-
-    @property
-    def alive(self) -> bool:
-        return self == Status.ALIVE
-
-    @property
-    def deleted(self) -> bool:
-        return self == Status.DELETED
-
-
 def _exit_if_response_error(response, expected_status):
     if response.status != expected_status:
         if response.status == HTTPStatus.FORBIDDEN:
-            _exit_error('[b]WOLF_TOKEN[/b] is not valid. Please check or login again.')
+            _exit_error(
+                'You are not logged in, please login using [b]jcloud login[/b] first.'
+            )
         else:
             _exit_error(
                 f'Bad response: expecting [b]{expected_status}[/b], got [b]{response.status}:{response.reason}[/b] from server.'
@@ -69,13 +44,11 @@ class CloudFlow:
     name: Optional[str] = None
     workspace_id: Optional[str] = None
     flow_id: Optional[str] = None
+    env_file: Optional[str] = None
 
     def __post_init__(self):
         from .auth import Auth
 
-        # check auth header
-        # if 'WOLF_TOKEN' not in os.environ:
-        #     print('[red][b]WOLF_TOKEN[/b] can not be found, please login first.[/red]')
         token = Auth.get_auth_token()
         if not token:
             _exit_error(
@@ -103,9 +76,45 @@ class CloudFlow:
     def _loop(self):
         return get_or_reuse_loop()
 
-    async def _zip_and_upload(self) -> str:
-        with zipdir(directory=Path(self.path)) as zipfilepath:
-            return await self._upload_project(filepaths=[zipfilepath])
+    @property
+    def artifact_metadata(self) -> Dict:
+        _path = Path(self.path)
+        _tags = {'filename': _path.name if _path.is_file() else 'flow.yml'}
+        if self.env_file is not None:
+            _env_path = Path(self.env_file)
+            if _env_path.exists():
+                logger.info(f'Passing env variables from {self.env_file} file')
+                _tags.update({'envfile': _env_path.name})
+            else:
+                _exit_error(f'env file [b]{self.env_file}[/b] not found')
+        else:
+            _env_path = _path / '.env'
+            if _env_path.exists():
+                logger.info(f'Passing env variables from default .env file ')
+                _tags.update({'envfile': _env_path.name})
+        return _tags
+
+    @property
+    def envs(self) -> Dict:
+        if self.env_file is not None:
+            _env_path = Path(self.env_file)
+            if _env_path.exists():
+                from dotenv import dotenv_values
+
+                return dict(dotenv_values(_env_path))
+            else:
+                _exit_error(f'env file [b]{self.env_file}[/b] not found')
+        else:
+            return {}
+
+    async def _zip_and_upload(self, directory: Path) -> str:
+        # extra steps for normalizing and normalized
+        pbar.update(pb_task, total=7)
+        with zipdir(directory=directory) as zipfilepath:
+            return await self._upload_project(
+                filepaths=[zipfilepath],
+                metadata=self.artifact_metadata,
+            )
 
     async def _get_post_params(self):
         params, _post_kwargs = {}, {}
@@ -114,15 +123,28 @@ class CloudFlow:
         if self.workspace_id:
             params['workspace'] = self.workspace_id
 
+        _data = aiohttp.FormData()
         _path = Path(self.path)
         if not _path.exists():
             _exit_error(f'Path {self.path} doesn\'t exist.')
         elif _path.is_dir():
-            pbar.update(pb_task, total=7)  # extra steps for normalizing and normalized
-            params['artifactid'] = await self._zip_and_upload()
+            _flow_path = _path / 'flow.yml'
+            if _flow_path.exists() and normalized(_flow_path):
+                _data.add_field(name='yaml', value=open(_flow_path))
+            else:
+                params['artifactid'] = await self._zip_and_upload(_path)
         elif _path.is_file():
-            _post_kwargs['data'] = {'yaml': open(self.path)}
+            if normalized(_path):
+                _data.add_field(name='yaml', value=open(_path))
+            else:
+                # normalize & deploy parent directory
+                params['artifactid'] = await self._zip_and_upload(_path.parent)
 
+        if self.envs:
+            _data.add_field(name='envs', value=json.dumps(self.envs))
+
+        if _data._fields:
+            _post_kwargs['data'] = _data
         _post_kwargs['params'] = params
         return _post_kwargs
 
@@ -189,9 +211,9 @@ class CloudFlow:
                     '\nYou don\'t have any Flows deployed. Please use [b]jc deploy[/b]'
                 )
 
-    async def _upload_project(self, filepaths: List[Path], tags: Dict = {}) -> str:
+    async def _upload_project(self, filepaths: List[Path], metadata: Dict = {}) -> str:
         data = aiohttp.FormData()
-        data.add_field(name='metaData', value=json.dumps(tags))
+        data.add_field(name='metaData', value=json.dumps(metadata))
         [
             data.add_field(
                 name='file', value=open(file.absolute(), 'rb'), filename=file.name
@@ -216,8 +238,11 @@ class CloudFlow:
     ):
         _wait_seconds = 0
         _last_status = None
-        while _wait_seconds < 600:
+        while _wait_seconds < 1800:
             _json_response = await self.status
+            if _json_response is None or 'status' not in _json_response:
+                # intermittently no response is sent, retry then!
+                continue
             _current_status = Status(_json_response['status'])
             if _last_status is None:
                 _last_status = _current_status
@@ -240,8 +265,12 @@ class CloudFlow:
                     advance=1,
                 )
             else:
-                await asyncio.sleep(1)
-                _wait_seconds += 1
+                await asyncio.sleep(5)
+                _wait_seconds += 5
+
+        _exit_error(
+            f'Couldn\'t reach status {desired} after waiting for 30mins. Exiting.'
+        )
 
     async def _terminate(self):
         async with aiohttp.ClientSession() as session:
@@ -270,28 +299,42 @@ class CloudFlow:
     async def logstream(params):
         logger.debug(f'Asked to stream logs with params {params}')
 
-        log_msg = logger.debug if 'request_id' in params else print
+        def dim_print(text):
+            print(f'[dim]{text}[/dim]')
+
+        log_msg = dim_print if 'request_id' in params else print
 
         try:
             async with aiohttp.ClientSession() as session:
-                try:
-                    async with session.ws_connect(LOGSTREAM_API, params=params) as ws:
-                        logger.debug(
-                            f'Successfully connected to logstream API with params: {params}'
+                while True:
+                    try:
+                        async with session.ws_connect(
+                            LOGSTREAM_API, params=params
+                        ) as ws:
+                            logger.debug(
+                                f'Successfully connected to logstream API with params: {params}'
+                            )
+                            await ws.send_json({})
+                            async for msg in ws:
+                                if msg.type == aiohttp.http.WSMsgType.TEXT:
+                                    log_dict: Dict = msg.json()
+                                    if (
+                                        log_dict.get('status') == 'STREAMING'
+                                        and logger.getEffectiveLevel() < logging.INFO
+                                    ):
+                                        log_msg(log_dict['message'])
+                        if 'request_id' in params:
+                            # NOTE: For the case of `request_id`, Flow deployment might be taking >15mins,
+                            # after which websocket endpoint will timeout. Hence reconnect & stream.
+                            continue
+                        else:
+                            logger.debug(f'Disconnected from the logstream server ...')
+                    except aiohttp.WSServerHandshakeError as e:
+                        logger.critical(
+                            f'Couldn\'t connect to the logstream server as {e!r}'
                         )
-                        await ws.send_json({})
-                        async for msg in ws:
-                            if msg.type == aiohttp.http.WSMsgType.TEXT:
-                                log_dict: Dict = msg.json()
-                                if log_dict.get('status') == 'STREAMING':
-                                    log_msg(log_dict['message'])
-                    logger.debug(f'Disconnected from the logstream server ...')
-                except aiohttp.WSServerHandshakeError as e:
-                    logger.critical(
-                        f'Couldn\'t connect to the logstream server as {e!r}'
-                    )
         except asyncio.CancelledError:
-            logger.debug(f'logstream task cancelled.')
+            logger.debug(f'Cancelling the logstreaming...')
         except Exception as e:
             logger.error(f'Got an exception while streaming logs {e!r}')
 
@@ -302,11 +345,11 @@ class CloudFlow:
                 pb_task,
                 advance=1,
                 description='Submitting',
-                title=f'Deploying {self.path}',
+                title=f'Deploying {Path(self.path).resolve()}',
             )
             await self._deploy()
             pbar.update(pb_task, description='Queueing (can take ~1 minute)', advance=1)
-            self.gateway = await self._fetch_until(
+            self.gateway: str = await self._fetch_until(
                 intermediate=[
                     Status.SUBMITTED,
                     Status.NORMALIZING,
@@ -319,10 +362,11 @@ class CloudFlow:
             pbar.update(pb_task, description='Finishing', advance=1)
             self._c_logstream_task.cancel()
 
-        # ask feedback
-        from .auth import Survey
+        if 'JCLOUD_NO_SURVEY' not in os.environ:
+            # ask feedback
+            from .auth import Survey
 
-        Survey().count().ask(threshold=3)
+            Survey().count().ask(threshold=3)
         return self
 
     async def __aexit__(self, *args, **kwargs):
